package name.nathanmcrae.numbersstation;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import java.io.File;
import java.io.IOException;
import java.time.format.DateTimeFormatter;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.Optional;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.Event;
import javafx.fxml.FXML;
import javafx.scene.control.CheckBox;
import javafx.scene.control.DatePicker;
import javafx.scene.control.ListView;
import javafx.scene.control.RadioButton;
import javafx.scene.control.Spinner;
import javafx.scene.control.SpinnerValueFactory;
import javafx.scene.control.TextField;
import javafx.scene.control.TextInputDialog;
import javafx.scene.control.ToggleGroup;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.KeyCode;
import javafx.scene.Node;
import javafx.stage.Stage;

public class MainSettingsController {
    private IntegerProperty digitsPerGroup = new SimpleIntegerProperty();
    private StringProperty externalProgramCommand = new SimpleStringProperty();
    private BooleanProperty manageScheduleExternally = new SimpleBooleanProperty();
    private IntegerProperty messageLength = new SimpleIntegerProperty();
    private ObjectProperty<StationSettings.MessageMethod> messageMethod = new SimpleObjectProperty<>();
    private ObjectProperty<StationSettings.MessagePeriod> messagePeriod = new SimpleObjectProperty<>();
    private StringProperty password = new SimpleStringProperty();
    private StationSettings settings;
    private StringProperty stationAddress = new SimpleStringProperty();
    private StringProperty stationName = new SimpleStringProperty();
    private ObjectProperty<LocalDate> scheduleStartDate = new SimpleObjectProperty<>();
    private StringProperty scheduleStartTime = new SimpleStringProperty();
    private StringProperty username = new SimpleStringProperty();

    @FXML
    private TextField stationNameField;

    @FXML
    private TextField stationAddressField;

    @FXML
    private RadioButton dailyRadioButton;

    @FXML
    private Spinner<Integer> digitsPerGroupSpinner;

    @FXML
    private TextField externalProgramCommandField;

    @FXML
    private CheckBox manageScheduleExternallyCheckBox;

    @FXML
    private Spinner<Integer> messageLengthSpinner;

    @FXML
    private ToggleGroup messageMethodGroup;

    @FXML
    private ToggleGroup messagePeriodGroup;

    @FXML
    private RadioButton monthlyRadioButton;

    @FXML
    private RadioButton ftpRadioButton;

    @FXML
    private RadioButton sftpRadioButton;

    @FXML
    private RadioButton scpRadioButton;

    @FXML
    private TextField scheduleStartTimeField;

    @FXML
    private RadioButton externalProgramRadioButton;

    @FXML
    private TextField usernameField;

    @FXML
    private TextField passwordField;

    @FXML
    private ListView<String> prefixListView;

    @FXML
    private DatePicker scheduleStartDatePicker;

    @FXML
    private RadioButton weeklyRadioButton;

    public MainSettingsController() { }

    @FXML
    private void initialize() {
        externalProgramCommandField.textProperty().bindBidirectional(externalProgramCommand);
        manageScheduleExternallyCheckBox.selectedProperty().bindBidirectional(manageScheduleExternally);
        stationNameField.textProperty().bindBidirectional(stationName);
        stationAddressField.textProperty().bindBidirectional(stationAddress);
        passwordField.textProperty().bindBidirectional(password);
        scheduleStartDatePicker.valueProperty().bindBidirectional(scheduleStartDate);
        scheduleStartTimeField.textProperty().bindBidirectional(scheduleStartTime);
        usernameField.textProperty().bindBidirectional(username);

        digitsPerGroupSpinner.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(1, 100, 1));
        messageLengthSpinner.setValueFactory(new SpinnerValueFactory.IntegerSpinnerValueFactory(1, 1000, 1));

        digitsPerGroupSpinner.getValueFactory().valueProperty().bindBidirectional(digitsPerGroup.asObject());
        messageLengthSpinner.getValueFactory().valueProperty().bindBidirectional(messageLength.asObject());

        scheduleStartTimeField.addEventFilter(KeyEvent.ANY, event -> {
            int cursorPosition = scheduleStartTimeField.getCaretPosition();

            String character = event.getCharacter();
            KeyCode code = event.getCode();

            String str = event.getText();

            // Consume the event to block the default behavior
            if (!(code == KeyCode.LEFT ||
                  code == KeyCode.RIGHT ||
                  code == KeyCode.UP ||
                  code == KeyCode.DOWN ||
                  code == KeyCode.HOME ||
                  code == KeyCode.END ||
                  code == KeyCode.PAGE_UP ||
                  code == KeyCode.PAGE_DOWN)) {
                event.consume();
            }

            if (event.getEventType() == KeyEvent.KEY_PRESSED &&
                (code == KeyCode.DIGIT0 ||
                 code == KeyCode.DIGIT1 ||
                 code == KeyCode.DIGIT2 ||
                 code == KeyCode.DIGIT3 ||
                 code == KeyCode.DIGIT4 ||
                 code == KeyCode.DIGIT5 ||
                 code == KeyCode.DIGIT6 ||
                 code == KeyCode.DIGIT7 ||
                 code == KeyCode.DIGIT8 ||
                 code == KeyCode.DIGIT9)) {

                 String editedTime = scheduleStartTimeField.getText();

                if (cursorPosition < scheduleStartTimeField.getLength()) {
                    char currentChar = scheduleStartTimeField.getText().charAt(cursorPosition);
                    if (currentChar == ':') {
                        cursorPosition++;
                    }
                    editedTime = editedTime.substring(0, cursorPosition) + str + editedTime.substring(cursorPosition + 1);
                }

                // Validate the time format
                String[] timeParts = editedTime.split(":");
                if (timeParts.length == 3) {
                    int hours = Integer.parseInt(timeParts[0]);
                    int minutes = Integer.parseInt(timeParts[1]);
                    int seconds = Integer.parseInt(timeParts[2]);

                    if (!(hours < 0 || hours > 23 || minutes < 0 || minutes > 59 || seconds < 0 || seconds > 59)) {
                        scheduleStartTimeField.setText(editedTime);
                        scheduleStartTimeField.positionCaret(cursorPosition + 1);
                    }
                }
            }
        });

        messageMethodGroup.selectedToggleProperty().addListener((observable, oldValue, newValue) -> {
            if (newValue == ftpRadioButton) {
                messageMethod.set(StationSettings.MessageMethod.FTP);
            } else if (newValue == sftpRadioButton) {
                messageMethod.set(StationSettings.MessageMethod.SFTP);
            } else if (newValue == scpRadioButton) {
                messageMethod.set(StationSettings.MessageMethod.SCP);
            } else if (newValue == externalProgramRadioButton) {
                messageMethod.set(StationSettings.MessageMethod.EXTERNAL_PROGRAM);
            }
        });

        messageMethod.addListener((observable, oldValue, newValue) -> {
            if (newValue == StationSettings.MessageMethod.FTP) {
                messageMethodGroup.selectToggle(ftpRadioButton);
            } else if (newValue == StationSettings.MessageMethod.SFTP) {
                messageMethodGroup.selectToggle(sftpRadioButton);
            } else if (newValue == StationSettings.MessageMethod.SCP) {
                messageMethodGroup.selectToggle(scpRadioButton);
            } else if (newValue == StationSettings.MessageMethod.EXTERNAL_PROGRAM) {
                messageMethodGroup.selectToggle(externalProgramRadioButton);
            }
        });

        messagePeriodGroup.selectedToggleProperty().addListener((observable, oldValue, newValue) -> {
            if (newValue == dailyRadioButton) {
                messagePeriod.set(StationSettings.MessagePeriod.DAILY);
            } else if (newValue == monthlyRadioButton) {
                messagePeriod.set(StationSettings.MessagePeriod.MONTHLY);
            } else if (newValue == weeklyRadioButton) {
                messagePeriod.set(StationSettings.MessagePeriod.WEEKLY);
            } 
        });

        messagePeriod.addListener((observable, oldValue, newValue) -> {
            if (newValue == StationSettings.MessagePeriod.DAILY) {
                messagePeriodGroup.selectToggle(dailyRadioButton);
            } else if (newValue == StationSettings.MessagePeriod.MONTHLY) {
                messagePeriodGroup.selectToggle(monthlyRadioButton);
            } else if (newValue == StationSettings.MessagePeriod.WEEKLY) {
                messagePeriodGroup.selectToggle(weeklyRadioButton);
            } 
        });

        // Set user data. Default result is no update.
        stationNameField.sceneProperty().addListener((observable, oldScene, newScene) -> {
            if (newScene != null) {
                Stage stage = (Stage) newScene.getWindow();
                stage.setUserData(false);
            }
        });
    }

    @FXML
    private void handleAddPrefixButtonPress() {
        TextInputDialog dialog = new TextInputDialog();
        dialog.setTitle("New prefix");
        dialog.setHeaderText("Add a new prefix");
        dialog.setContentText("Enter a message prefix / identifier:");

        Optional<String> result = dialog.showAndWait();
        result.ifPresent(prefix -> {
            prefixListView.getItems().add(prefix);
        });
    }

    @FXML
    private void handleCancelButtonPress(Event e) {
        Node node = (Node) e.getSource();
        Stage stage = (Stage) node.getScene().getWindow();
        stage.close();
    }

    @FXML
    private void handleRemovePrefixButtonPress() {
        int selectedIndex = prefixListView.getSelectionModel().getSelectedIndex();
        if (selectedIndex >= 0) {
            prefixListView.getItems().remove(selectedIndex);
        }
    }

    @FXML
    private void handleSaveButtonPress(Event e) {
        settings.setAddress(stationAddress.get());
        settings.setDigitsPerGroup(digitsPerGroup.get());
        settings.setExternalProgramCommand(externalProgramCommand.get());
        settings.setManageScheduleExternally(manageScheduleExternally.get());
        settings.setMessageLength(messageLength.get());
        settings.setMessageMethod(messageMethod.get());
        settings.setMessagePeriod(messagePeriod.get());
        settings.setName(stationName.get());
        settings.setPassword(password.get());
        settings.getPrefixes().clear();
        settings.getPrefixes().addAll(prefixListView.getItems());
        settings.setScheduleStartDate(scheduleStartDatePicker.getValue());
        settings.setUsername(username.get());

        try {
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("HH:mm:ss");
            LocalTime startTime = LocalTime.parse(scheduleStartTimeField.getText(), formatter);
            settings.setScheduleStartTime(startTime);
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        Node node = (Node) e.getSource();
        Stage stage = (Stage) node.getScene().getWindow();
        stage.setUserData(true);
        stage.close();
    }

    @FXML
    private void handleTestConnectionButtonPress() {
        // TODO
    }

    public StringProperty stationAddressProperty() {
        return stationAddress;
    }

    public void setStationSettings(StationSettings newSettings) {
        settings = newSettings;

        stationAddress.set(settings.getAddress());
        stationName.set(settings.getName());
        digitsPerGroup.set(settings.getDigitsPerGroup());
        externalProgramCommand.set(settings.getExternalProgramCommand());
        manageScheduleExternally.set(settings.getManageScheduleExternally());
        messageLength.set(settings.getMessageLength());
        messageMethod.set(settings.getMessageMethod());
        messagePeriod.set(settings.getMessagePeriod());
        username.set(settings.getUsername());
        password.set(settings.getPassword());
        System.out.println(settings.getPrefixes());
        prefixListView.getItems().addAll(settings.getPrefixes());
        scheduleStartDate.set(settings.getScheduleStartDate());
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("HH:mm:ss");
        LocalTime startTime = settings.getScheduleStartTime();
        if (startTime == null) {
            startTime = LocalTime.now();
        }
        scheduleStartTime.set(startTime.format(formatter));
    }
}
